@@ -66,10 +66,6 @@
 #include "gvfsdnssdresolver.h"
 #endif
 
-/* Overwrite maximal number of connections that libsoup can open in one time in
-   order to prevent backend lockups when too many files is opened concurrently. */
-#define MAX_CONNS 32
-
 typedef struct _MountAuthData MountAuthData;
 
 static void mount_auth_info_free (MountAuthData *info);
@@ -1113,6 +1109,13 @@ ms_response_to_file_info (MsResponse *response,
       icon = g_themed_icon_new ("folder");
       symbolic_icon = g_themed_icon_new ("folder-symbolic");
       file_info_set_content_type (info, "inode/directory");
+
+      /* Ignore file size for directories. Most of the servers don't report it
+       * for directories anyway. However, some servers report total size of
+       * files inside the directory, which is not expected and causes issues
+       * for clients.
+       */
+      g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
     }
   else
     {
@@ -1872,11 +1875,7 @@ do_mount (GVfsBackend  *backend,
   G_VFS_BACKEND_HTTP (backend)->mount_base = mount_base; 
 
   /* Override the HTTP backend's default. */
-  g_object_set (session,
-                "ssl-strict", TRUE,
-                SOUP_SESSION_MAX_CONNS_PER_HOST, MAX_CONNS,
-                SOUP_SESSION_MAX_CONNS, MAX_CONNS,
-                NULL);
+  g_object_set (session, "ssl-strict", TRUE, NULL);
 
   data = &(G_VFS_BACKEND_DAV (backend)->auth_info); 
   data->mount_source = g_object_ref (mount_source);
